/*
* ASIX FORTE serial FLASH M25P80 example
* distributed as-is, without any warranties
* http://www.asix.net/
*
* version 1.0, Dec 2016
* 
* Programmer connection table: 
* M25P80  <->  FORTE
* S#      <->  P
* VCC     <->  VDD
* VSS     <->  GND
* DQ0     <->  D
* C       <->  C
* DQ1     <->  I
* W#      <->  VDD
* HOLD#   <->  VDD
*
*/

#include <stdio.h>
#include "forte.h"

void ReadStatus(char* message)
{
    int status = AGetBlocking();
    char* codestring = ReturnCodeString(status);
    if(message[0] != '\0') {
        printf("%s: %s\n", message, codestring);
    }
    else {
        printf("%s\n", codestring);
    }
}

void PrintHex(int* data, int count)
{
    int i;
    for(i=0;i<count;i++) {
        printf("%02X ", data[i]);
        if(((i+1)%16) == 0) printf("\n");
    }
    if((count%16) != 0) printf("\n");
}

unsigned char LsbToMsb(unsigned char databyte)
{
    int i;
    unsigned char x = 0;
    for(i=0;i<8;i++) {
        x = x | (((databyte >> i) & 1) << (7-i));
    }
    return x;
}

int main(int argc, char *argv[])
{
    int i, n, mem_bytes_count, status, received_bytes;
    int data_count, buffer_count, buffer_size, block_in_size;
    int *data, *buffer;
    
    printf("ASIX FORTE serial FLASH M25P80 example \n");
   
    if(LoadForteDll("forte.dll") == FORTE_DLL_LOAD_OK) {
        printf("DLL load OK\n");
    }
    else {
        printf("DLL load FAILED\n");
        return 1;
    }
    
    QOpenProg(-1);
    ReadStatus("");
    
    // initiate
    QSetGPIOAnswer(0);
    ReadStatus("SetGPIOAnswer");
    QSetActiveLED(LED_ACT_Y_FAST_BLINK);
    ReadStatus("SetActiveLED");
    QPoweronVdd(20000, 3300); // supply voltage 3.3V
    ReadStatus("PoweronVdd");
    QSetShiftSpeed(SHIFT_CLK_10000kHz);
    QSetPins((PINS_HI << PINS_P_BIT) | (PINS_LO << PINS_C_BIT) | (PINS_LO << PINS_D_BIT)); // S#=1, C=0, DQ0=0
    QDelay(100);
    QSetPins((PINS_LO << PINS_P_BIT)); // S#=0
    QDelay(1);
    
    // command code and address
    QShiftByte(0xC0, SHIFT_MODE0);
    QShiftByte(0x00, SHIFT_MODE0);
    QShiftByte(0x00, SHIFT_MODE0);
    QShiftByte(0x00, SHIFT_MODE0);
    // read memory
    mem_bytes_count = 1024*1024; // total read bytes
    block_in_size = 512; // bytes in one QShiftBytes_In()
    buffer_size = 16384; // bytes in one AGetBlock()
    printf("Reading %d bytes ...\n", mem_bytes_count);
    data = (int *) malloc(mem_bytes_count * sizeof(int));
    data_count = 0;
    buffer = (int *) malloc(buffer_size * sizeof(int));
    while(data_count < mem_bytes_count) {
        buffer_count = 0;
        while((buffer_count < buffer_size) && (data_count+buffer_count < mem_bytes_count)) {
            n = mem_bytes_count - data_count - buffer_count;
            if(n > block_in_size) n = block_in_size;
            QShiftBytes_In(SHIFT_MODE0, SHIFT_OUTIN_PINI, n);
            buffer_count += n;
        }
        while(1) {
            status = AGetStatus(&received_bytes);
            if(!status) break;
            if(received_bytes >= buffer_count) break;
        }
        if(!status) break;
        AGetBlock(buffer, buffer_count, &received_bytes);
        for(i=0;i<buffer_count;i++) data[data_count+i] = buffer[i];
        data_count += buffer_count;
    }
    //
    QSetPins((PINS_HI << PINS_P_BIT)); // S#=1
    QSetActiveLED(LED_ACT_OFF);
    ReadStatus("SetActiveLED");

    // take 8-bit data and print
    if(status) {
        for(i=0;i<mem_bytes_count;i++) data[i] = LsbToMsb(data[i] & 0xFF);
        printf("First 128 data bytes:\n");
        PrintHex(data, 128);
        printf("Last 128 data bytes:\n");
        PrintHex(&(data[mem_bytes_count-128]), 128);
    }
    else {
        printf("Data read failed\n");
    }

    QSetGPIOAnswer(1);
    ReadStatus("SetGPIOAnswer");
    QCloseProg();
    ReadStatus("");
    free(data);
    free(buffer);
    
    return 0;
}

